'=========================================================================
'          Huge Integer Math and Encryption library dll
'                   ( H I M E . d l l )
'                 (C) 2002-2007 DevOTechS
'                 e-mail: support@devotechs.com
'
'         Demo program to use HIME.dll from Liberty Basic.
'               HIME.dll can be downloaded from:
'                      www.devotechs.com
'
'=========================================================================


        'Open HIME.dll
    OPEN "HIME.dll" for dll as #hime
    print "Initialising HIME..."

        'Define some structs because HIME parameters needs to be called by reference.
        'Liberty Basic needs structs to be able to do this.
    struct LongByRef1, i as long
    struct LongByRef2, i as long
    struct LongByRef3, i as long
    struct LongByRef4, i as long

    LongByRef1.i.struct = 1
    LongByRef2.i.struct = 2
    LongByRef3.i.struct = 3
    LongByRef4.i.struct = 4

        '1) Show version number of current HIME.dll
    CALLDLL #hime, "HI_VERSION", result as long
    print "1) HIME.dll Version: " ; result
    print ""


        '2) Register HIME.dll in case you have purchased a HIME license key.
        'This will suppress the 10s nagscreen:
    i = HiPut("xxx-yyyyyyyyy", 1)
    CALLDLL #hime, "HI_REGISTER", LongByRef1 as struct, result as long
    print "2) License key, read from register: " ; HiGet$(1)
    print ""


        '3) Store a number in a HIME register and do some calculations with it
    i = HiPut("220", 1)             'Store number in register 1
    i = HiPut("100", 2)             'Store number in register 2

    LongByRef1.i.struct = 1         'Set register numbers in order to use it for calling
    LongByRef2.i.struct = 2         'HIME function

                                    'Convert from decimal format to 'huge integer' format
    CALLDLL #hime, "HI_DEC2HUGE" , LongByRef1 as struct, LongByRef1 as struct, result as long
    CALLDLL #hime, "HI_DEC2HUGE" , LongByRef2 as struct, LongByRef2 as struct, result as long

                                   'Multiply register 2 by 2
    CALLDLL #hime, "HI_MULBY2"   , LongByRef2 as struct, LongByRef2 as struct, result as long

                                    'Increment register 1 2 times
    CALLDLL #hime, "HI_INCR"     , LongByRef1 as struct, LongByRef1 as struct, result as long
    CALLDLL #hime, "HI_INCR"     , LongByRef1 as struct, LongByRef1 as struct, result as long

                                    'Add 2 registers. Register 1 = Register 1 + Register 2
    CALLDLL #hime, "HI_ADD"      , LongByRef1 as struct, LongByRef2 as struct, LongByRef1 as struct, result as long

                                    'Convert from huge integer to decimal for printing
    CALLDLL #hime, "HI_HUGE2DEC" , LongByRef1 as struct, LongByRef1 as struct, result as long
    print "3) Calculation result : " ; HiGet$(LongByRef1.i.struct)
    print ""


        '4) Concatenate a few bytes to register data
    print "4) Byte concatenation in a register "
    LongByRef1.i.struct = 1
                                    'Clear register 1 contents
    CALLDLL #hime, "HI_REGCLEAR" , LongByRef1 as struct, result as long
    CALLDLL #hime, "HI_REGGETLEN", LongByRef1 as struct, result as long
    print "Nr of bytes in register: " ; result

    for i = 1 to 4                  'Concatenate some bytes
        LongByRef2.i.struct = i * 10    'This is the byte to concatenate
        CALLDLL #hime, "HI_REGCONCATBYTE", LongByRef1 as struct, LongByRef2 as struct, result as long
        CALLDLL #hime, "HI_REGGETLEN", LongByRef1 as struct, result as long
        print "Nr of bytes in register: " ; result
    next i

                                    'Get a single byte from a register
    LongByRef2.i.struct = 3         'Let's retrieve the 3rd byte
    CALLDLL #hime, "HI_GETREGBYTE", LongByRef1 as struct, LongByRef2 as struct, result as long
    print "The retrieved byte is " ; result
    print ""



        '5) AES encryption and decryption
    print "5) AES encryption and decryption"
    LongByRef1.i.struct = 1
    LongByRef2.i.struct = 2
    LongByRef3.i.struct = 3
    LongByRef4.i.struct = 4

        '5a) Encryption
        'Store plaintext in register 1
    i = HiPut("This is my secret message. Can be as long as I want!", 1)   'Plaintext can have any length

        'Store encryption key in register 2
    i = HiPut("my secret key---", 2)   'Must be exactly 16, 24 or 32 bytes long!
              '1234567890123456

        'Set an empty IV in register 3
    i = HiPut("", 3)                   'Empty IV. HIME will generate one itself

        'Encrypt plaintext
    CALLDLL #hime, "HI_ENCRYPT_AES_CBC", LongByRef1 as struct,_
                                         LongByRef2 as struct,_
                                         LongByRef3 as struct,_
                                         LongByRef4 as struct,_
                                         result as long
                          'Register 4 now contains the ciphertext
    if result <> 0 then print "encryption error #" ; result
    print "ciphertext: " ; HiGet$(4)


        '5b) Decryption
        'Move ciphertext to register 1
    CALLDLL #hime, "HI_COPYREG",         LongByRef4 as struct,_
                                         LongByRef1 as struct,_
                                         result as long

        'Store decryption key in register 2
    i = HiPut("my secret key---", 2)  'Must be exactly 16, 24 or 32 bytes long
          '1234567890123456

        'Decrypt ciphertext
    CALLDLL #hime, "HI_DECRYPT_AES_CBC", LongByRef1 as struct,_
                                         LongByRef2 as struct,_
                                         LongByRef3 as struct,_
                                         result as long
                    'Register 3 now contains decrypted ciphertext (= plaintext)
    if result <> 0 then print "decryption error #" ; result
    print "Decrypted data: " ; HiGet$(3)

    CALLDLL #hime, "HI_REGREVERSE",      LongByRef3 as struct,_
                                         LongByRef3 as struct,_
                                         result as long
    print "Reverse the text, just for fun..:"
    print "Text reversed: " ; HiGet$(3)

        'Unload HIME.dll
    close #hime

    print "Done"
    end
'------------------------------------------------------------

'------------------------------------------------------------
'Put data in a HIME register:
'INPUT: Dat$: data to be stored in a HIME register
'       Reg : number of the register to store data in
'FUNCTION: The function returns a (possible) error value. Or 0 in case
'       everything was okay.
function HiPut(Dat$, Reg)
    struct LongByRef, i as long

    LongByRef.i.struct = Reg
    CALLDLL #hime, "HI_PUTREGASCIIZ", Dat$ as ptr, LongByRef as struct, result as long
    HiPut = result
end function
'------------------------------------------------------------

'------------------------------------------------------------
'Get data from a HIME register:
'INPUT: Reg : number of the register to get data from
'FUNCTION: The function returns a string with the data from the register
function HiGet$(Reg)
    struct LongByRef, i as long

    LongByRef.i.struct = Reg
    CALLDLL #hime, "HI_REGGETLEN", LongByRef as struct, result as long
    result2$ = space$(result)
    CALLDLL #hime, "HI_GETREGASCIIZ2", LongByRef as struct, result2$ as ptr, result as long
    HiGet$ = result2$ 
end function
'------------------------------------------------------------
